@@ -4,7 +4,7 @@
 **
 ** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
 **
-** Contact: Nokia Corporation (info@qt.nokia.com)
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 **
 ** GNU Lesser General Public License Usage
@@ -26,7 +26,7 @@
 ** conditions contained in a signed written agreement between you and Nokia.
 **
 ** If you have questions regarding the use of this file, please contact
-** Nokia at info@qt.nokia.com.
+** Nokia at qt-info@nokia.com.
 **
 **************************************************************************/
 
@@ -68,8 +68,7 @@ static const QRegExp &windowsDeviceSubDirPattern()
 // ----------- FileNameValidatingLineEdit
 FileNameValidatingLineEdit::FileNameValidatingLineEdit(QWidget *parent) :
     BaseValidatingLineEdit(parent),
-    m_allowDirectories(false),
-    m_unused(0)
+    m_allowDirectories(false)
 {
 }
 
@@ -91,8 +90,8 @@ void FileNameValidatingLineEdit::setAllowDirectories(bool v)
 #  define SLASHES "/"
 #endif
 
-static const char notAllowedCharsSubDir[]   = "~!?:&*\"|#%<> ";
-static const char notAllowedCharsNoSubDir[] = "~!?:&*\"|#%<> "SLASHES;
+static const char notAllowedCharsSubDir[]   = "~!?:&*\"|#%<>$\"'();`' ";
+static const char notAllowedCharsNoSubDir[] = "~!?:&*\"|#%<>$\"'();`' "SLASHES;
 
 static const char *notAllowedSubStrings[] = {".."};
 
@@ -143,7 +142,46 @@ bool FileNameValidatingLineEdit::validateFileName(const QString &name,
 
 bool  FileNameValidatingLineEdit::validate(const QString &value, QString *errorMessage) const
 {
-    return validateFileName(value, m_allowDirectories, errorMessage);
+    return validateFileNameExtension(value, requiredExtensions(), errorMessage)
+            && validateFileName(value, allowDirectories(), errorMessage);
+}
+
+
+bool FileNameValidatingLineEdit::validateFileNameExtension(const QString &fileName,
+                                                           const QStringList &requiredExtensions,
+                                                           QString *errorMessage)
+{
+    // file extension
+    if (!requiredExtensions.isEmpty()) {
+        foreach (const QString &requiredExtension, requiredExtensions) {
+            QString extension = QLatin1String(".") + requiredExtension;
+            if (fileName.endsWith(extension, Qt::CaseSensitive) && extension.count() < fileName.count()) {
+                return true;
+            }
+        }
+
+        if (errorMessage) {
+            if (requiredExtensions.count() == 1) {
+                *errorMessage = tr("File extension %1 is required:").arg(requiredExtensions.first());
+            } else {
+                *errorMessage = tr("File extensions %1 are required:").arg(requiredExtensions.join(", "));
+            }
+        }
+
+        return false;
+    }
+
+    return true;
+}
+
+QStringList FileNameValidatingLineEdit::requiredExtensions() const
+{
+    return m_requiredExtensionList;
+}
+
+void FileNameValidatingLineEdit::setRequiredExtensions(const QStringList &extensions)
+{
+    m_requiredExtensionList = extensions;
 }
 
 } // namespace Utils
