@@ -4,7 +4,7 @@
 **
 ** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
 **
-** Contact: Nokia Corporation (info@qt.nokia.com)
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 **
 ** GNU Lesser General Public License Usage
@@ -50,7 +50,7 @@ class AttachGdbAdapter : public AbstractGdbAdapter
     Q_OBJECT
 
 public:
-    explicit AttachGdbAdapter(GdbEngine *engine, QObject *parent = 0);
+    explicit AttachGdbAdapter(GdbEngine *engine);
 
 private:
     DumperHandling dumperHandling() const { return DumperLoadedByGdb; }
