@@ -2,28 +2,31 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
-** Commercial Usage
-**
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
 **
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this file.
+** Please review the following information to ensure the GNU Lesser General
+** Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** If you are unsure which license is appropriate for your use, please
-** contact the sales department at http://qt.nokia.com/contact.
+** In addition, as a special exception, Nokia gives you certain additional
+** rights. These rights are described in the Nokia Qt LGPL Exception
+** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+**
+** Other Usage
+**
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+** If you have questions regarding the use of this file, please contact
+** Nokia at qt-info@nokia.com.
 **
 **************************************************************************/
 
@@ -61,9 +64,12 @@ void GitSubmitEditorPanelData::clear()
 QString GitSubmitEditorPanelData::authorString() const
 {
     QString rc;
-    rc += QLatin1Char('"');
     rc += author;
-    rc += QLatin1String("\" <");
+
+    if (email.isEmpty())
+        return rc;
+
+    rc += QLatin1String(" <");
     rc += email;
     rc += QLatin1Char('>');
     return rc;
@@ -79,6 +85,7 @@ void CommitData::clear()
 {
     panelInfo.clear();
     panelData.clear();
+    amendSHA1.clear();
 
     stagedFiles.clear();
     unstagedFiles.clear();
@@ -139,6 +146,7 @@ bool CommitData::parseFilesFromStatus(const QString &output)
     const QString branchIndicator = QLatin1String(kBranchIndicatorC);
     const QString commitIndicator = QLatin1String("# Changes to be committed:");
     const QString notUpdatedIndicator = QLatin1String("# Changed but not updated:");
+    const QString notUpdatedIndicatorGit174 = QLatin1String("# Changes not staged for commit:");
     const QString untrackedIndicator = QLatin1String("# Untracked files:");
 
     State s = None;
@@ -148,39 +156,43 @@ bool CommitData::parseFilesFromStatus(const QString &output)
 
     const QStringList::const_iterator cend = lines.constEnd();
     for (QStringList::const_iterator it =  lines.constBegin(); it != cend; ++it) {
-        const QString line = *it;
+        QString line = *it;
         if (line.startsWith(branchIndicator)) {
             panelInfo.branch = line.mid(branchIndicator.size() + 1);
-        } else {
-            if (line.startsWith(commitIndicator)) {
-                s = CommitFiles;
-            } else {
-                if (line.startsWith(notUpdatedIndicator)) {
-                    s = NotUpdatedFiles;
-                } else {
-                    if (line.startsWith(untrackedIndicator)) {
-                        // Now match untracked: "#<tab>foo.cpp"
-                        s = UntrackedFiles;
-                        filesPattern = QRegExp(QLatin1String("#\\t.+"));
-                        QTC_ASSERT(filesPattern.isValid(), return false);
-                    } else {
-                        if (filesPattern.exactMatch(line)) {
-                            switch (s) {
-                            case CommitFiles:
-                                addStateFileSpecification(line, &stagedFiles);
-                            break;
-                            case NotUpdatedFiles:
-                                addStateFileSpecification(line, &unstagedFiles);
-                                break;
-                            case UntrackedFiles:
-                                untrackedFiles.push_back(line.mid(2).trimmed());
-                                break;
-                            case None:
-                                break;
-                            }
-                        }
-                    }
-                }
+            continue;
+        }
+        if (line.startsWith(commitIndicator)) {
+            s = CommitFiles;
+            continue;
+        }
+        if (line.startsWith(notUpdatedIndicator) || line.startsWith(notUpdatedIndicatorGit174)) {
+            s = NotUpdatedFiles;
+            continue;
+        }
+        if (line.startsWith(untrackedIndicator)) {
+            // Now match untracked: "#<tab>foo.cpp"
+            s = UntrackedFiles;
+            filesPattern = QRegExp(QLatin1String("#\\t.+"));
+            QTC_ASSERT(filesPattern.isValid(), return false);
+            continue;
+        }
+        if (filesPattern.exactMatch(line)) {
+            switch (s) {
+            case CommitFiles:
+                addStateFileSpecification(line, &stagedFiles);
+                break;
+            case NotUpdatedFiles:
+                // skip submodules:
+                if (line.endsWith(QLatin1String(" (modified content)"))
+                        || line.endsWith(" (new commits)"))
+                    line = line.left(line.lastIndexOf(QLatin1Char('(')) - 1);
+                addStateFileSpecification(line, &unstagedFiles);
+                break;
+            case UntrackedFiles:
+                untrackedFiles.push_back(line.mid(2).trimmed());
+                break;
+            case None:
+                break;
             }
         }
     }
