@@ -2,28 +2,31 @@
 **
 ** This file is part of Qt Creator
 **
-** Copyright (c) 2009 Nokia Corporation and/or its subsidiary(-ies).
+** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
-** Commercial Usage
-**
-** Licensees holding valid Qt Commercial licenses may use this file in
-** accordance with the Qt Commercial License Agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Nokia.
 **
 ** GNU Lesser General Public License Usage
 **
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this file.
+** Please review the following information to ensure the GNU Lesser General
+** Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** If you are unsure which license is appropriate for your use, please
-** contact the sales department at http://qt.nokia.com/contact.
+** In addition, as a special exception, Nokia gives you certain additional
+** rights. These rights are described in the Nokia Qt LGPL Exception
+** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+**
+** Other Usage
+**
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+** If you have questions regarding the use of this file, please contact
+** Nokia at qt-info@nokia.com.
 **
 **************************************************************************/
 
@@ -33,9 +36,11 @@
 #include "projectexplorerconstants.h"
 
 #include <coreplugin/mimedatabase.h>
+#include <coreplugin/fileiconprovider.h>
 #include <utils/qtcassert.h>
 
 #include <QtCore/QFileInfo>
+#include <QtCore/QDir>
 #include <QtGui/QApplication>
 #include <QtGui/QIcon>
 #include <QtGui/QStyle>
@@ -43,14 +48,20 @@
 using namespace ProjectExplorer;
 
 /*!
-  \class FileNode
+  \class ProjectExplorer::Node
 
-  Base class of all nodes in the node hierarchy.
+  \brief Base class of all nodes in the node hierarchy.
 
-  \see FileNode
-  \see FolderNode
-  \see ProjectNode
+  The nodes are arranged in a tree where leaves are FileNodes and non-leaves are FolderNodes
+  A Project is a special Folder that manages the files and normal folders underneath it.
+
+  The Watcher emits signals for structural changes in the hierarchy.
+  A Visitor can be used to traverse all Projects and other Folders.
+
+  \sa ProjectExplorer::FileNode, ProjectExplorer::FolderNode, ProjectExplorer::ProjectNode
+  \sa ProjectExplorer::NodesWatcher, ProjectExplorer::NodesVisitor
 */
+
 Node::Node(NodeType nodeType,
            const QString &filePath)
         : QObject(),
@@ -59,6 +70,7 @@ Node::Node(NodeType nodeType,
           m_folderNode(0),
           m_path(filePath)
 {
+
 }
 
 NodeType Node::nodeType() const
@@ -67,7 +79,7 @@ NodeType Node::nodeType() const
 }
 
 /*!
-  Project that owns & manages the node. It's the first project in list of ancestors.
+  \brief Project that owns & manages the node. It's the first project in list of ancestors.
   */
 ProjectNode *Node::projectNode() const
 {
@@ -75,7 +87,7 @@ ProjectNode *Node::projectNode() const
 }
 
 /*!
-  Parent in node hierarchy.
+  \brief Parent in node hierarchy.
   */
 FolderNode *Node::parentFolderNode() const
 {
@@ -83,7 +95,7 @@ FolderNode *Node::parentFolderNode() const
 }
 
 /*!
-  Path of file or folder in the filesystem the node represents.
+  \brief Path of file or folder in the filesystem the node represents.
   */
 QString Node::path() const
 {
@@ -111,12 +123,11 @@ void Node::setPath(const QString &path)
 }
 
 /*!
-  \class FileNode
+  \class ProjectExplorer::FileNode
 
-  In-memory presentation of a file. All FileNode's are leaf nodes.
+  \brief In-memory presentation of a file. All FileNode's are leaf nodes.
 
-  \see FolderNode
-  \see ProjectNode
+  \sa ProjectExplorer::FolderNode, ProjectExplorer::ProjectNode
 */
 
 FileNode::FileNode(const QString &filePath,
@@ -134,7 +145,7 @@ FileType FileNode::fileType() const
 }
 
 /*!
-  Returns true if the file is automatically generated by a compile step.
+  \brief Returns true if the file is automatically generated by a compile step.
   */
 bool FileNode::isGenerated() const
 {
@@ -142,22 +153,16 @@ bool FileNode::isGenerated() const
 }
 
 /*!
-  \class FolderNode
+  \class ProjectExplorer::FolderNode
 
   In-memory presentation of a folder. Note that the node itself + all children (files and folders) are "managed" by the owning project.
 
-  \see FileNode
-  \see ProjectNode
+  \sa ProjectExplorer::FileNode, ProjectExplorer::ProjectNode
 */
-FolderNode::FolderNode(const QString &folderPath)
-        : Node(FolderNodeType, folderPath),
-          m_folderName(folderPath)
+FolderNode::FolderNode(const QString &folderPath)  :
+    Node(FolderNodeType, folderPath),
+    m_displayName(QDir::toNativeSeparators(folderPath))
 {
-    static QIcon dirIcon;
-    if (dirIcon.isNull()) {
-        dirIcon = QApplication::style()->standardIcon(QStyle::SP_DirIcon);
-    }
-    m_icon = dirIcon;
 }
 
 FolderNode::~FolderNode()
@@ -166,23 +171,25 @@ FolderNode::~FolderNode()
     qDeleteAll(m_fileNodes);
 }
 
-/*
-  The display name that should be used in a view.
-
-
-  \see setFolderName()
+/*!
+    \brief The display name that should be used in a view.
+    \sa setFolderName()
  */
-QString FolderNode::name() const
+
+QString FolderNode::displayName() const
 {
-    return m_folderName;
+    return m_displayName;
 }
 
-/*
-  The icon that should be used in a view. Default is the directory icon (QStyle::S_PDirIcon).
-  \see setIcon()
+/*!
+  \brief The icon that should be used in a view. Default is the directory icon (QStyle::S_PDirIcon).
+  s\a setIcon()
  */
 QIcon FolderNode::icon() const
 {
+    // Instantiating the Icon provider is expensive.
+    if (m_icon.isNull())
+        m_icon = Core::FileIconProvider::instance()->icon(QFileIconProvider::Folder);
     return m_icon;
 }
 
@@ -203,9 +210,9 @@ void FolderNode::accept(NodesVisitor *visitor)
         subFolder->accept(visitor);
 }
 
-void FolderNode::setFolderName(const QString &name)
+void FolderNode::setDisplayName(const QString &name)
 {
-    m_folderName = name;
+    m_displayName = name;
 }
 
 void FolderNode::setIcon(const QIcon &icon)
@@ -214,17 +221,17 @@ void FolderNode::setIcon(const QIcon &icon)
 }
 
 /*!
-  \class ProjectNode
+  \class ProjectExplorer::ProjectNode
+
+  \brief In-memory presentation of a Project.
 
-  In-memory presentation of a Project.
   A concrete subclass must implement the "persistent" stuff
 
-  \see FileNode
-  \see FolderNode
+  \sa ProjectExplorer::FileNode, ProjectExplorer::FolderNode
 */
 
-/*
-  Creates uninitialized ProjectNode object.
+/*!
+  \brief Creates uninitialized ProjectNode object.
   */
 ProjectNode::ProjectNode(const QString &projectFilePath)
         : FolderNode(projectFilePath)
@@ -232,7 +239,7 @@ ProjectNode::ProjectNode(const QString &projectFilePath)
     setNodeType(ProjectNodeType);
     // project node "manages" itself
     setProjectNode(this);
-    setFolderName(QFileInfo(m_folderName).fileName());
+    setDisplayName(QFileInfo(projectFilePath).fileName());
 }
 
 QList<ProjectNode*> ProjectNode::subProjectNodes() const
@@ -240,6 +247,18 @@ QList<ProjectNode*> ProjectNode::subProjectNodes() const
     return m_subProjectNodes;
 }
 
+void ProjectNode::aboutToChangeHasBuildTargets()
+{
+    foreach (NodesWatcher *watcher, watchers())
+        emit watcher->aboutToChangeHasBuildTargets(this);
+}
+
+void ProjectNode::hasBuildTargetsChanged()
+{
+    foreach (NodesWatcher *watcher, watchers())
+        emit watcher->hasBuildTargetsChanged(this);
+}
+
 /*!
   \function bool ProjectNode::addSubProjects(const QStringList &)
   */
@@ -260,15 +279,23 @@ QList<ProjectNode*> ProjectNode::subProjectNodes() const
   \function bool ProjectNode::renameFile(const FileType, const QString &, const QString &)
   */
 
+bool ProjectNode::deploysFolder(const QString &folder) const
+{
+    Q_UNUSED(folder);
+    return false;
+}
+
 QList<NodesWatcher*> ProjectNode::watchers() const
 {
     return m_watchers;
 }
 
-/*
-   Registers a watcher for the current project & all sub projects
+/*!
+   \brief Registers a watcher for the current project & all sub projects
+
    It does not take ownership of the watcher.
-   */
+*/
+
 void ProjectNode::registerWatcher(NodesWatcher *watcher)
 {
     if (!watcher)
@@ -280,8 +307,8 @@ void ProjectNode::registerWatcher(NodesWatcher *watcher)
         subProject->registerWatcher(watcher);
 }
 
-/*
-  Removes a watcher for the current project & all sub projects.
+/*!
+    \brief Removes a watcher for the current project & all sub projects.
 */
 void ProjectNode::unregisterWatcher(NodesWatcher *watcher)
 {
@@ -301,7 +328,7 @@ void ProjectNode::accept(NodesVisitor *visitor)
 }
 
 /*!
-  Adds project nodes to the hierarchy and emits the corresponding signals.
+  \brief Adds project nodes to the hierarchy and emits the corresponding signals.
   */
 void ProjectNode::addProjectNodes(const QList<ProjectNode*> &subProjects)
 {
@@ -314,8 +341,8 @@ void ProjectNode::addProjectNodes(const QList<ProjectNode*> &subProjects)
             emit watcher->foldersAboutToBeAdded(this, folderNodes);
 
         foreach (ProjectNode *project, subProjects) {
-            QTC_ASSERT(!project->parentFolderNode(),
-                qDebug("Project node has already a parent"));
+            QTC_ASSERT(!project->parentFolderNode() || project->parentFolderNode() == this,
+                       qDebug("Project node has already a parent"));
             project->setParentFolderNode(this);
             foreach (NodesWatcher *watcher, m_watchers)
                 project->registerWatcher(watcher);
@@ -333,9 +360,11 @@ void ProjectNode::addProjectNodes(const QList<ProjectNode*> &subProjects)
 }
 
 /*!
-  Remove project nodes from the hierarchy and emits the corresponding signals.
+  \brief Remove project nodes from the hierarchy and emits the corresponding signals.
+
   All objects in the argument list are deleted.
-  */
+*/
+
 void ProjectNode::removeProjectNodes(const QList<ProjectNode*> &subProjects)
 {
     if (!subProjects.isEmpty()) {
@@ -372,8 +401,8 @@ void ProjectNode::removeProjectNodes(const QList<ProjectNode*> &subProjects)
 }
 
 /*!
-  Adds folder nodes to the hierarchy and emits the corresponding signals.
-  */
+  \brief Adds folder nodes to the hierarchy and emits the corresponding signals.
+*/
 void ProjectNode::addFolderNodes(const QList<FolderNode*> &subFolders, FolderNode *parentFolder)
 {
     Q_ASSERT(parentFolder);
@@ -422,9 +451,10 @@ void ProjectNode::addFolderNodes(const QList<FolderNode*> &subFolders, FolderNod
 }
 
 /*!
-  Remove file nodes from the hierarchy and emits the corresponding signals.
+  \brief Remove file nodes from the hierarchy and emits the corresponding signals.
+
   All objects in the subFolders list are deleted.
-  */
+*/
 void ProjectNode::removeFolderNodes(const QList<FolderNode*> &subFolders,
                                    FolderNode *parentFolder)
 {
@@ -461,9 +491,11 @@ void ProjectNode::removeFolderNodes(const QList<FolderNode*> &subFolders,
 }
 
 /*!
-  Adds file nodes to the internal list and emits the corresponding signals.
+  \brief Adds file nodes to the internal list and emits the corresponding signals.
+
   This method should be called within an implementation of the public method addFiles.
-  */
+*/
+
 void ProjectNode::addFileNodes(const QList<FileNode*> &files, FolderNode *folder)
 {
     Q_ASSERT(folder);
@@ -508,10 +540,12 @@ void ProjectNode::addFileNodes(const QList<FileNode*> &files, FolderNode *folder
 }
 
 /*!
-  Remove file nodes from the internal list and emits the corresponding signals.
+  \brief Remove file nodes from the internal list and emits the corresponding signals.
+
   All objects in the argument list are deleted.
   This method should be called within an implementation of the public method removeFiles.
-  */
+*/
+
 void ProjectNode::removeFileNodes(const QList<FileNode*> &files, FolderNode *folder)
 {
     Q_ASSERT(folder);
@@ -551,19 +585,14 @@ void ProjectNode::watcherDestroyed(QObject *watcher)
 }
 
 /*!
-  Sort pointers to FileNodes
-  */
+  \brief Sort pointers to FileNodes
+*/
 bool ProjectNode::sortNodesByPath(Node *n1, Node *n2) {
     return n1->path() < n2->path();
 }
 
-bool ProjectNode::sortFolderNodesByName(FolderNode *f1, FolderNode *f2)
-{
-    return f1->name() < f2->name();
-}
-
 /*!
-  \class SessionNode
+  \class ProjectExplorer::SessionNode
 */
 
 SessionNode::SessionNode(const QString &sessionPath, QObject *parentObject)
@@ -578,10 +607,11 @@ QList<NodesWatcher*> SessionNode::watchers() const
     return m_watchers;
 }
 
-/*
-   Registers a watcher for the complete session tree.
+/*!
+   \brief Registers a watcher for the complete session tree.
    It does not take ownership of the watcher.
 */
+
 void SessionNode::registerWatcher(NodesWatcher *watcher)
 {
     if (!watcher)
@@ -593,9 +623,10 @@ void SessionNode::registerWatcher(NodesWatcher *watcher)
         project->registerWatcher(watcher);
 }
 
-/*
-    Removes a watcher from the complete session tree
+/*!
+    \brief Removes a watcher from the complete session tree
 */
+
 void SessionNode::unregisterWatcher(NodesWatcher *watcher)
 {
     if (!watcher)
@@ -682,9 +713,9 @@ void SessionNode::watcherDestroyed(QObject *watcher)
 }
 
 /*!
-  \class NodesWatcher
+  \class ProjectExplorer::NodesWatcher
 
-  NodesWatcher let you keep track of changes in the tree.
+  \brief NodesWatcher lets you keep track of changes in the tree.
 
   Add a watcher by calling ProjectNode::registerWatcher() or
   SessionNode::registerWatcher(). Whenever the tree underneath the
@@ -694,8 +725,10 @@ void SessionNode::watcherDestroyed(QObject *watcher)
   by calling ProjectNode::unregisterWatcher and
   SessionNode::unregisterWatcher().
 
-  The NodesWatcher is similar to the Observer in the
+  The NodesWatcher is similar to the Observer in the  
   well-known Observer pattern (Booch et al).
+
+  \sa ProjectExplorer::Node
 */
 
 NodesWatcher::NodesWatcher(QObject *parent)
@@ -721,5 +754,7 @@ FileType typeForFileName(const Core::MimeDatabase *db, const QFileInfo &file)
         return ResourceType;
     if (typeName == QLatin1String(Constants::FORM_MIMETYPE))
         return FormType;
+    if (typeName == QLatin1String(Constants::QML_MIMETYPE))
+        return QMLType;
     return UnknownFileType;
 }
