@@ -4,7 +4,7 @@
 **
 ** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
 **
-** Contact: Nokia Corporation (info@qt.nokia.com)
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 **
 ** GNU Lesser General Public License Usage
@@ -26,18 +26,21 @@
 ** conditions contained in a signed written agreement between you and Nokia.
 **
 ** If you have questions regarding the use of this file, please contact
-** Nokia at info@qt.nokia.com.
+** Nokia at qt-info@nokia.com.
 **
 **************************************************************************/
 
 #include "qt4nodes.h"
 #include "qt4project.h"
+#include "qt4target.h"
 #include "qt4projectmanager.h"
 #include "qt4projectmanagerconstants.h"
 #include "qtuicodemodelsupport.h"
 #include "qmakestep.h"
+#include "qt4buildconfiguration.h"
 
 #include <projectexplorer/nodesvisitor.h>
+#include <projectexplorer/runconfiguration.h>
 
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/editormanager/ieditor.h>
@@ -85,22 +88,22 @@ struct FileTypeDataStorage {
 
 static const FileTypeDataStorage fileTypeDataStorage[] = {
     { ProjectExplorer::HeaderType,
-      QT_TRANSLATE_NOOP("Qt4ProjectManager::Internal::Qt4PriFileNode", "Headers"),
+      QT_TRANSLATE_NOOP("Qt4ProjectManager::Qt4PriFileNode", "Headers"),
       ":/qt4projectmanager/images/headers.png" },
     { ProjectExplorer::SourceType,
-      QT_TRANSLATE_NOOP("Qt4ProjectManager::Internal::Qt4PriFileNode", "Sources"),
+      QT_TRANSLATE_NOOP("Qt4ProjectManager::Qt4PriFileNode", "Sources"),
       ":/qt4projectmanager/images/sources.png" },
     { ProjectExplorer::FormType,
-      QT_TRANSLATE_NOOP("Qt4ProjectManager::Internal::Qt4PriFileNode", "Forms"),
+      QT_TRANSLATE_NOOP("Qt4ProjectManager::Qt4PriFileNode", "Forms"),
       ":/qt4projectmanager/images/forms.png" },
     { ProjectExplorer::ResourceType,
-      QT_TRANSLATE_NOOP("Qt4ProjectManager::Internal::Qt4PriFileNode", "Resources"),
+      QT_TRANSLATE_NOOP("Qt4ProjectManager::Qt4PriFileNode", "Resources"),
       ":/qt4projectmanager/images/qt_qrc.png" },
     { ProjectExplorer::QMLType,
-      QT_TRANSLATE_NOOP("Qt4ProjectManager::Internal::Qt4PriFileNode", "QML"),
+      QT_TRANSLATE_NOOP("Qt4ProjectManager::Qt4PriFileNode", "QML"),
       ":/qt4projectmanager/images/qml.ico" }, // TODO icon
     { ProjectExplorer::UnknownFileType,
-      QT_TRANSLATE_NOOP("Qt4ProjectManager::Internal::Qt4PriFileNode", "Other files"),
+      QT_TRANSLATE_NOOP("Qt4ProjectManager::Qt4PriFileNode", "Other files"),
       ":/qt4projectmanager/images/unknown.png" }
 };
 
@@ -137,7 +140,7 @@ Q_GLOBAL_STATIC_WITH_INITIALIZER(Qt4NodeStaticData, qt4NodeStaticData, {
                                                     overlayIcon, desiredSize);
         QIcon folderIcon;
         folderIcon.addPixmap(folderPixmap);
-        const QString desc = Qt4ProjectManager::Internal::Qt4PriFileNode::tr(fileTypeDataStorage[i].typeName);
+        const QString desc = Qt4ProjectManager::Qt4PriFileNode::tr(fileTypeDataStorage[i].typeName);
         x->fileTypeData.push_back(Qt4NodeStaticData::FileTypeData(fileTypeDataStorage[i].type,
                                                                   desc, folderIcon));
     }
@@ -159,8 +162,8 @@ static void clearQt4NodeStaticData()
 
 enum { debug = 0 };
 
-namespace Qt4ProjectManager {
-namespace Internal {
+using namespace Qt4ProjectManager;
+using namespace Qt4ProjectManager::Internal;
 
 Qt4PriFile::Qt4PriFile(Qt4PriFileNode *qt4PriFile)
     : IFile(qt4PriFile), m_priFile(qt4PriFile)
@@ -245,7 +248,8 @@ Qt4PriFileNode::Qt4PriFileNode(Qt4Project *project, Qt4ProFileNode* qt4ProFileNo
           m_project(project),
           m_qt4ProFileNode(qt4ProFileNode),
           m_projectFilePath(QDir::fromNativeSeparators(filePath)),
-          m_projectDir(QFileInfo(filePath).absolutePath())
+          m_projectDir(QFileInfo(filePath).absolutePath()),
+          m_includedInExactParse(true)
 {
     Q_ASSERT(project);
     m_qt4PriFile = new Qt4PriFile(this);
@@ -262,6 +266,8 @@ void Qt4PriFileNode::scheduleUpdate()
     m_qt4ProFileNode->scheduleUpdate();
 }
 
+namespace Qt4ProjectManager {
+namespace Internal {
 struct InternalNode
 {
     QMap<QString, InternalNode*> subnodes;
@@ -478,7 +484,8 @@ struct InternalNode
             projectNode->addFileNodes(filesToAdd, folder);
     }
 };
-
+}
+}
 
 QStringList Qt4PriFileNode::baseVPaths(QtSupport::ProFileReader *reader, const QString &projectDir)
 {
@@ -552,7 +559,7 @@ void Qt4PriFileNode::update(ProFile *includeFileExact, QtSupport::ProFileReader
     for (int i=0; i < folders.size(); ++i) {
         const QFileInfo fi(folders.at(i));
         if (fi.isRelative())
-            folders[i] = projectDir + '/' + folders.at(i);
+            folders[i] = QDir::cleanPath(projectDir + '/' + folders.at(i));
     }
 
 
@@ -653,9 +660,9 @@ void Qt4PriFileNode::watchFolders(const QSet<QString> &folders)
     toWatch.subtract(m_watchedFolders);
 
     if (!toUnwatch.isEmpty())
-        m_project->centralizedFolderWatcher()->unwatchFolders(toUnwatch.toList(), this);
+        m_project->unwatchFolders(toUnwatch.toList(), this);
     if (!toWatch.isEmpty())
-        m_project->centralizedFolderWatcher()->watchFolders(toWatch.toList(), this);
+        m_project->watchFolders(toWatch.toList(), this);
 
     m_watchedFolders = folders;
 }
@@ -726,6 +733,7 @@ void Qt4PriFileNode::folderChanged(const QString &folder)
 
     contents.updateSubFolders(this, this);
     m_project->updateFileList();
+    m_project->updateCodeModels();
 
     // The files to be packaged are listed inside the symbian build system.
     // We need to regenerate that list by running qmake
@@ -765,6 +773,27 @@ QList<ProjectExplorer::RunConfiguration *> Qt4PriFileNode::runConfigurationsFor(
     return m_project->activeTarget()->runConfigurationsForNode(node);
 }
 
+QList<Qt4PriFileNode *> Qt4PriFileNode::subProjectNodesExact() const
+{
+    QList<Qt4PriFileNode *> nodes;
+    foreach (ProjectNode *node, subProjectNodes()) {
+        Qt4PriFileNode *n = qobject_cast<Qt4PriFileNode *>(node);
+        if (n && n->includedInExactParse())
+            nodes << n;
+    }
+    return nodes;
+}
+
+bool Qt4PriFileNode::includedInExactParse() const
+{
+    return m_includedInExactParse;
+}
+
+void Qt4PriFileNode::setIncludedInExactParse(bool b)
+{
+    m_includedInExactParse = b;
+}
+
 QList<ProjectNode::ProjectAction> Qt4PriFileNode::supportedActions(Node *node) const
 {
     QList<ProjectAction> actions;
@@ -975,7 +1004,7 @@ bool Qt4PriFileNode::priFileWritable(const QString &path)
     switch (Core::FileManager::promptReadOnlyFile(path, versionControl, core->mainWindow(), false)) {
     case Core::FileManager::RO_OpenVCS:
         if (!versionControl->vcsOpen(path)) {
-            QMessageBox::warning(core->mainWindow(), tr("Cannot Open File"), tr("Cannot open the file for edit with VCS."));
+            QMessageBox::warning(core->mainWindow(), tr("Cannot Open File"), tr("Cannot open the file for editing with VCS."));
             return false;
         }
         break;
@@ -1143,10 +1172,7 @@ void Qt4PriFileNode::changeFiles(const FileType fileType,
 void Qt4PriFileNode::save(const QStringList &lines)
 {
     Utils::FileSaver saver(m_projectFilePath, QIODevice::Text);
-    foreach (const QString &str, lines) {
-        saver.write(str.toLocal8Bit());
-        saver.write("\n", 1);
-    }
+    saver.write(lines.join(QLatin1String("\n")).toLocal8Bit());
     saver.finalize(Core::ICore::instance()->mainWindow());
 
     m_project->qt4ProjectManager()->notifyChanged(m_projectFilePath);
@@ -1306,9 +1332,6 @@ Qt4NodesWatcher::Qt4NodesWatcher(QObject *parent)
 {
 }
 
-} // namespace Internal
-
-
 const Qt4ProFileNode *Qt4ProFileNode::findProFileFor(const QString &fileName) const
 {
     if (fileName == path())
@@ -1398,7 +1421,6 @@ Qt4ProFileNode::Qt4ProFileNode(Qt4Project *project,
           m_readerExact(0),
           m_readerCumulative(0)
 {
-
     if (parent)
         setParent(parent);
 
@@ -1464,7 +1486,7 @@ QStringList Qt4ProFileNode::variableValue(const Qt4Variable var) const
     return m_varValues.value(var);
 }
 
-void Qt4ProFileNode::emitProFileUpdated()
+void Qt4ProFileNode::emitProFileUpdatedRecursive()
 {
     foreach (ProjectExplorer::NodesWatcher *watcher, watchers())
         if (Internal::Qt4NodesWatcher *qt4Watcher = qobject_cast<Internal::Qt4NodesWatcher*>(watcher))
@@ -1472,26 +1494,36 @@ void Qt4ProFileNode::emitProFileUpdated()
 
     foreach (ProjectNode *subNode, subProjectNodes()) {
         if (Qt4ProFileNode *node = qobject_cast<Qt4ProFileNode *>(subNode)) {
-            node->emitProFileUpdated();
+            node->emitProFileUpdatedRecursive();
         }
     }
 }
 
+void Qt4ProFileNode::setParseInProgressRecursive(bool b)
+{
+    setParseInProgress(b);
+    foreach (ProjectNode *subNode, subProjectNodes()) {
+        if (Qt4ProFileNode *node = qobject_cast<Qt4ProFileNode *>(subNode)) {
+            node->setParseInProgressRecursive(b);
+        }
+    }
+}
+
+void Qt4ProFileNode::setParseInProgress(bool b)
+{
+    if (m_parseInProgress == b)
+        return;
+    m_parseInProgress = b;
+    foreach (ProjectExplorer::NodesWatcher *watcher, watchers())
+        if (Internal::Qt4NodesWatcher *qt4Watcher = qobject_cast<Internal::Qt4NodesWatcher*>(watcher))
+            emit qt4Watcher->proFileUpdated(this, m_validParse, m_parseInProgress);
+}
+
 bool Qt4ProFileNode::validParse() const
 {
     return m_validParse;
 }
 
-void Qt4ProFileNode::setParseInProgressRecursive()
-{
-    m_parseInProgress = true;
-    foreach (ProjectNode *subNode, subProjectNodes()) {
-        if (Qt4ProFileNode *node = qobject_cast<Qt4ProFileNode *>(subNode)) {
-            node->setParseInProgressRecursive();
-        }
-    }
-}
-
 bool Qt4ProFileNode::parseInProgress() const
 {
     return m_parseInProgress;
@@ -1499,8 +1531,7 @@ bool Qt4ProFileNode::parseInProgress() const
 
 void Qt4ProFileNode::scheduleUpdate()
 {
-    setParseInProgressRecursive();
-    emitProFileUpdated();
+    setParseInProgressRecursive(true);
     m_project->scheduleAsyncUpdate(this);
 }
 
@@ -1515,11 +1546,7 @@ void Qt4ProFileNode::asyncUpdate()
 
 void Qt4ProFileNode::update()
 {
-    m_parseInProgress = true;
-    foreach (ProjectExplorer::NodesWatcher *watcher, watchers())
-        if (Internal::Qt4NodesWatcher *qt4Watcher = qobject_cast<Internal::Qt4NodesWatcher*>(watcher))
-            emit qt4Watcher->proFileUpdated(this, m_validParse, m_parseInProgress);
-
+    setParseInProgressRecursive(true);
     setupReader();
     EvalResult evalResult = evaluate();
     applyEvaluate(evalResult, false);
@@ -1575,16 +1602,14 @@ void Qt4ProFileNode::applyEvaluate(EvalResult evalResult, bool async)
             m_project->proFileParseError(tr("Error while parsing file %1. Giving up.").arg(m_projectFilePath));
             invalidate();
         }
-        foreach (ProjectExplorer::NodesWatcher *watcher, watchers())
-            if (Internal::Qt4NodesWatcher *qt4Watcher = qobject_cast<Internal::Qt4NodesWatcher*>(watcher))
-                emit qt4Watcher->proFileUpdated(this, false, false);
+        setParseInProgressRecursive(false);
         return;
     }
 
     if (debug)
         qDebug() << "Qt4ProFileNode - updating files for file " << m_projectFilePath;
 
-    Qt4ProjectType projectType = Internal::proFileTemplateTypeToProjectType(
+    Qt4ProjectType projectType = proFileTemplateTypeToProjectType(
                 (evalResult == EvalOk ? m_readerExact : m_readerCumulative)->templateType());
     if (projectType != m_projectType) {
         Qt4ProjectType oldType = m_projectType;
@@ -1615,10 +1640,13 @@ void Qt4ProFileNode::applyEvaluate(EvalResult evalResult, bool async)
 
     QStringList newProjectFilesExact;
     QHash<QString, ProFile*> includeFilesExact;
+    QSet<QString> exactSubdirs;
     ProFile *fileForCurrentProjectExact = 0;
     if (evalResult == EvalOk) {
-        if (m_projectType == SubDirsTemplate)
+        if (m_projectType == SubDirsTemplate) {
             newProjectFilesExact = subDirsPaths(m_readerExact);
+            exactSubdirs = newProjectFilesExact.toSet();
+        }
         foreach (ProFile *includeFile, m_readerExact->includeFiles()) {
             if (includeFile->fileName() == m_projectFilePath) { // this file
                 fileForCurrentProjectExact = includeFile;
@@ -1715,15 +1743,20 @@ void Qt4ProFileNode::applyEvaluate(EvalResult evalResult, bool async)
             ProFile *fileExact = includeFilesExact.value((*existingIt)->path());
             ProFile *fileCumlative = includeFilesCumlative.value((*existingIt)->path());
             if (fileExact || fileCumlative) {
-                static_cast<Qt4PriFileNode *>(*existingIt)->update(fileExact, m_readerExact, fileCumlative, m_readerCumulative);
+                Qt4PriFileNode *priFileNode = static_cast<Qt4PriFileNode *>(*existingIt);
+                priFileNode->update(fileExact, m_readerExact, fileCumlative, m_readerCumulative);
+                priFileNode->setIncludedInExactParse(fileExact != 0 && includedInExactParse());
             } else {
                 // We always parse exactly, because we later when async parsing don't know whether
                 // the .pro file is included in this .pro file
                 // So to compare that later parse with the sync one
+                Qt4ProFileNode *proFileNode = static_cast<Qt4ProFileNode *>(*existingIt);
+                // TODO that could be made faster...
+                proFileNode->setIncludedInExactParse(exactSubdirs.contains(proFileNode->path()) && includedInExactParse());
                 if (async)
-                    static_cast<Qt4ProFileNode *>(*existingIt)->asyncUpdate();
+                    proFileNode->asyncUpdate();
                 else
-                    static_cast<Qt4ProFileNode *>(*existingIt)->update();
+                    proFileNode->update();
             }
             ++existingIt;
             // newCumalativeIt and newExactIt are already incremented
@@ -1749,11 +1782,13 @@ void Qt4ProFileNode::applyEvaluate(EvalResult evalResult, bool async)
             } else if (fileExact || fileCumlative) {
                 Qt4PriFileNode *qt4PriFileNode = new Qt4PriFileNode(m_project, this, nodeToAdd);
                 qt4PriFileNode->setParentFolderNode(this); // Needed for loop detection
+                qt4PriFileNode->setIncludedInExactParse(fileExact != 0 && includedInExactParse());
                 qt4PriFileNode->update(fileExact, m_readerExact, fileCumlative, m_readerCumulative);
                 toAdd << qt4PriFileNode;
             } else {
                 Qt4ProFileNode *qt4ProFileNode = new Qt4ProFileNode(m_project, nodeToAdd);
                 qt4ProFileNode->setParentFolderNode(this); // Needed for loop detection
+                qt4ProFileNode->setIncludedInExactParse(exactSubdirs.contains(qt4ProFileNode->path()) && includedInExactParse());
                 if (async)
                     qt4ProFileNode->asyncUpdate();
                 else
@@ -1775,6 +1810,7 @@ void Qt4ProFileNode::applyEvaluate(EvalResult evalResult, bool async)
 
         // update TargetInformation
         m_qt4targetInformation = targetInformation(m_readerExact);
+        m_resolvedMkspecPath = m_project->proFileOption()->qmakespec;
 
         setupInstallsList(m_readerExact);
         setupProjectVersion(m_readerExact);
@@ -1798,6 +1834,7 @@ void Qt4ProFileNode::applyEvaluate(EvalResult evalResult, bool async)
                     QLatin1String("QML_IMPORT_PATH"), m_projectDir);
         newVarValues[Makefile] = m_readerExact->values("MAKEFILE");
         newVarValues[SymbianCapabilities] = m_readerExact->values("TARGET.CAPABILITY");
+        newVarValues[QtVar] = m_readerExact->values(QLatin1String("QT"));
 
         m_isDeployable = false;
         if (m_projectType == ApplicationTemplate) {
@@ -1821,15 +1858,11 @@ void Qt4ProFileNode::applyEvaluate(EvalResult evalResult, bool async)
         }
     } // evalResult == EvalOk
 
-    m_parseInProgress = false;
+    setParseInProgress(false);
 
     createUiCodeModelSupport();
     updateUiFiles();
 
-    foreach (ProjectExplorer::NodesWatcher *watcher, watchers())
-        if (Internal::Qt4NodesWatcher *qt4Watcher = qobject_cast<Internal::Qt4NodesWatcher*>(watcher))
-            emit qt4Watcher->proFileUpdated(this, true, false);
-
     m_project->destroyProFileReader(m_readerExact);
     m_project->destroyProFileReader(m_readerCumulative);
 
@@ -1850,7 +1883,7 @@ QStringList Qt4ProFileNode::updateUiFiles()
         return QStringList();
 
     // Find all ui files
-    Internal::FindUiFileNodesVisitor uiFilesVisitor;
+    FindUiFileNodesVisitor uiFilesVisitor;
     this->accept(&uiFilesVisitor);
     const QList<ProjectExplorer::FileNode*> uiFiles = uiFilesVisitor.uiFileNodes;
 
@@ -2027,7 +2060,7 @@ QStringList Qt4ProFileNode::subDirsPaths(QtSupport::ProFileReader *reader) const
         }
 
         if (QFile::exists(realFile)) {
-            subProjectPaths << realFile;
+            subProjectPaths << QDir::cleanPath(realFile);
         } else {
             m_project->proFileParseError(tr("Could not find .pro file for sub dir '%1' in '%2'")
                                          .arg(subDirVar).arg(realDir));
@@ -2086,7 +2119,7 @@ TargetInformation Qt4ProFileNode::targetInformation(QtSupport::ProFileReader *re
         // Hmm can we find out whether it's debug or release in a saner way?
         // Theoretically it's in CONFIG
         QString qmakeBuildConfig = "release";
-        if (m_project->activeTarget()->activeBuildConfiguration()->qmakeBuildConfiguration() & QtSupport::BaseQtVersion::DebugBuild)
+        if (m_project->activeTarget()->activeQt4BuildConfiguration()->qmakeBuildConfiguration() & QtSupport::BaseQtVersion::DebugBuild)
             qmakeBuildConfig = "debug";
         wd += QLatin1Char('/') + qmakeBuildConfig;
     }
@@ -2106,6 +2139,11 @@ TargetInformation Qt4ProFileNode::targetInformation() const
     return m_qt4targetInformation;
 }
 
+QString Qt4ProFileNode::resolvedMkspecPath() const
+{
+    return m_resolvedMkspecPath;
+}
+
 void Qt4ProFileNode::setupInstallsList(const QtSupport::ProFileReader *reader)
 {
     m_installsList.clear();
@@ -2213,7 +2251,7 @@ QString Qt4ProFileNode::buildDir(Qt4BuildConfiguration *bc) const
     const QDir srcDirRoot = QFileInfo(m_project->rootProjectNode()->path()).absoluteDir();
     const QString relativeDir = srcDirRoot.relativeFilePath(m_projectDir);
     if (!bc && m_project->activeTarget())
-        bc = m_project->activeTarget()->activeBuildConfiguration();
+        bc = m_project->activeTarget()->activeQt4BuildConfiguration();
     if (!bc)
         return QString();
     return QDir(bc->buildDirectory()).absoluteFilePath(relativeDir);
@@ -2294,7 +2332,7 @@ void Qt4ProFileNode::createUiCodeModelSupport()
     // Only those two project types can have ui files for us
     if (m_projectType == ApplicationTemplate || m_projectType == LibraryTemplate) {
         // Find all ui files
-        Internal::FindUiFileNodesVisitor uiFilesVisitor;
+        FindUiFileNodesVisitor uiFilesVisitor;
         this->accept(&uiFilesVisitor);
         const QList<ProjectExplorer::FileNode*> uiFiles = uiFilesVisitor.uiFileNodes;
 
@@ -2326,5 +2364,3 @@ void Qt4ProFileNode::createUiCodeModelSupport()
         delete it.value();
     }
 }
-
-} // namespace Qt4ProjectManager
