@@ -4,7 +4,7 @@
 **
 ** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
 **
-** Contact: Nokia Corporation (info@qt.nokia.com)
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 **
 ** GNU Lesser General Public License Usage
@@ -32,8 +32,8 @@
 
 #include "qt4project.h"
 
-#include "profilereader.h"
 #include "qt4projectmanager.h"
+#include "qt4target.h"
 #include "makestep.h"
 #include "qmakestep.h"
 #include "qt4nodes.h"
@@ -42,9 +42,10 @@
 #include "projectloadwizard.h"
 #include "qt4buildconfiguration.h"
 #include "findqt4profiles.h"
-#include "qmldumptool.h"
 
 #include <coreplugin/icore.h>
+#include <coreplugin/ifile.h>
+#include <coreplugin/icontext.h>
 #include <coreplugin/messagemanager.h>
 #include <coreplugin/coreconstants.h>
 #include <coreplugin/progressmanager/progressmanager.h>
@@ -53,13 +54,20 @@
 #include <qmljs/qmljsmodelmanagerinterface.h>
 #include <projectexplorer/toolchain.h>
 #include <projectexplorer/headerpath.h>
+#include <projectexplorer/target.h>
 #include <projectexplorer/buildenvironmentwidget.h>
 #include <projectexplorer/customexecutablerunconfiguration.h>
 #include <projectexplorer/projectexplorer.h>
+#include <projectexplorer/projectexplorerconstants.h>
 #include <utils/qtcassert.h>
+#include <qtsupport/qmldumptool.h>
+#include <qtsupport/baseqtversion.h>
+#include <qtsupport/profilereader.h>
+#include <qtsupport/qtsupportconstants.h>
 
 #include <QtCore/QDebug>
 #include <QtCore/QDir>
+#include <QtCore/QFileSystemWatcher>
 #include <QtGui/QFileDialog>
 #include <QtGui/QInputDialog>
 
@@ -72,6 +80,62 @@ enum { debug = 0 };
 namespace Qt4ProjectManager {
 namespace Internal {
 
+class Qt4ProjectFile : public Core::IFile
+{
+    Q_OBJECT
+
+public:
+    Qt4ProjectFile(Qt4Project *project, const QString &filePath, QObject *parent = 0);
+
+    bool save(QString *errorString, const QString &fileName, bool autoSave);
+    QString fileName() const;
+    virtual void rename(const QString &newName);
+
+    QString defaultPath() const;
+    QString suggestedFileName() const;
+    virtual QString mimeType() const;
+
+    bool isModified() const;
+    bool isReadOnly() const;
+    bool isSaveAsAllowed() const;
+
+    ReloadBehavior reloadBehavior(ChangeTrigger state, ChangeType type) const;
+    bool reload(QString *errorString, ReloadFlag flag, ChangeType type);
+
+private:
+    const QString m_mimeType;
+    Qt4Project *m_project;
+    QString m_filePath;
+};
+
+/// Watches folders for Qt4PriFile nodes
+/// use one file system watcher to watch all folders
+/// such minimizing system ressouce usage
+
+class CentralizedFolderWatcher : public QObject
+{
+    Q_OBJECT
+public:
+    CentralizedFolderWatcher(QObject *parent);
+    ~CentralizedFolderWatcher();
+    void watchFolders(const QList<QString> &folders, Qt4PriFileNode *node);
+    void unwatchFolders(const QList<QString> &folders, Qt4PriFileNode *node);
+
+private slots:
+    void folderChanged(const QString &folder);
+    void onTimer();
+    void delayedFolderChanged(const QString &folder);
+
+private:
+    QSet<QString> recursiveDirs(const QString &folder);
+    QFileSystemWatcher m_watcher;
+    QMultiMap<QString, Qt4PriFileNode *> m_map;
+
+    QSet<QString> m_recursiveWatchedFolders;
+    QTimer m_compressTimer;
+    QSet<QString> m_changedFolders;
+};
+
 // Qt4ProjectFiles: Struct for (Cached) lists of files in a project
 struct Qt4ProjectFiles {
     void clear();
@@ -119,8 +183,8 @@ QDebug operator<<(QDebug d, const  Qt4ProjectFiles &f)
 // A visitor to collect all files of a project in a Qt4ProjectFiles struct
 class ProjectFilesVisitor : public ProjectExplorer::NodesVisitor
 {
-    Q_DISABLE_COPY(ProjectFilesVisitor)
     ProjectFilesVisitor(Qt4ProjectFiles *files);
+
 public:
 
     static void findProjectFiles(Qt4ProFileNode *rootNode, Qt4ProjectFiles *files);
@@ -168,10 +232,10 @@ void ProjectFilesVisitor::visitFolderNode(FolderNode *folderNode)
     }
 }
 
-}
 }
 
 // ----------- Qt4ProjectFile
+namespace Internal {
 Qt4ProjectFile::Qt4ProjectFile(Qt4Project *project, const QString &filePath, QObject *parent)
     : Core::IFile(parent),
       m_mimeType(QLatin1String(Qt4ProjectManager::Constants::PROFILE_MIMETYPE)),
@@ -180,7 +244,7 @@ Qt4ProjectFile::Qt4ProjectFile(Qt4Project *project, const QString &filePath, QOb
 {
 }
 
-bool Qt4ProjectFile::save(const QString &)
+bool Qt4ProjectFile::save(QString *, const QString &, bool)
 {
     // This is never used
     return false;
@@ -236,12 +300,15 @@ Core::IFile::ReloadBehavior Qt4ProjectFile::reloadBehavior(ChangeTrigger state,
     return BehaviorSilent;
 }
 
-void Qt4ProjectFile::reload(ReloadFlag flag, ChangeType type)
+bool Qt4ProjectFile::reload(QString *errorString, ReloadFlag flag, ChangeType type)
 {
+    Q_UNUSED(errorString)
     Q_UNUSED(flag)
     Q_UNUSED(type)
+    return true;
 }
 
+} // namespace Internal
 /*!
   \class Qt4Project
 
@@ -258,8 +325,13 @@ Qt4Project::Qt4Project(Qt4Manager *manager, const QString& fileName) :
     m_asyncUpdateFutureInterface(0),
     m_pendingEvaluateFuturesCount(0),
     m_asyncUpdateState(NoState),
-    m_cancelEvaluate(false)
+    m_cancelEvaluate(false),
+    m_codeModelCanceled(false),
+    m_centralizedFolderWatcher(0)
 {
+    setProjectContext(Core::Context(Qt4ProjectManager::Constants::PROJECT_ID));
+    setProjectLanguage(Core::Context(ProjectExplorer::Constants::LANG_CXX));
+
     m_asyncUpdateTimer.setSingleShot(true);
     m_asyncUpdateTimer.setInterval(3000);
     connect(&m_asyncUpdateTimer, SIGNAL(timeout()), this, SLOT(asyncUpdate()));
@@ -299,6 +371,7 @@ bool Qt4Project::fromMap(const QVariantMap &map)
         if (t->buildConfigurations().isEmpty()) {
             qWarning() << "Removing" << t->id() << "since it has no buildconfigurations!";
             removeTarget(t);
+            delete t;
         }
     }
 
@@ -330,14 +403,11 @@ bool Qt4Project::fromMap(const QVariantMap &map)
     foreach (Target *t, targets())
         onAddedTarget(t);
 
-    connect(m_nodesWatcher, SIGNAL(proFileUpdated(Qt4ProjectManager::Internal::Qt4ProFileNode*,bool)),
-            this, SIGNAL(proFileUpdated(Qt4ProjectManager::Internal::Qt4ProFileNode *,bool)));
-
-    connect(m_nodesWatcher, SIGNAL(proFileInvalidated(Qt4ProjectManager::Internal::Qt4ProFileNode*)),
-            this, SIGNAL(proFileInvalidated(Qt4ProjectManager::Internal::Qt4ProFileNode*)));
+    connect(m_nodesWatcher, SIGNAL(proFileUpdated(Qt4ProjectManager::Qt4ProFileNode*,bool,bool)),
+            this, SIGNAL(proFileUpdated(Qt4ProjectManager::Qt4ProFileNode *,bool,bool)));
 
     // Now we emit update once :)
-    m_rootProjectNode->emitProFileUpdated();
+    m_rootProjectNode->emitProFileUpdatedRecursive();
 
 
     // Setup Qt versions supported (== possible targets).
@@ -413,7 +483,7 @@ void Qt4Project::updateCodeModels()
 
 void Qt4Project::updateCppCodeModel()
 {
-    Qt4BuildConfiguration *activeBC = activeTarget()->activeBuildConfiguration();
+    Qt4BuildConfiguration *activeBC = activeTarget()->activeQt4BuildConfiguration();
 
     CPlusPlus::CppModelManagerInterface *modelmanager =
         CPlusPlus::CppModelManagerInterface::instance();
@@ -426,7 +496,9 @@ void Qt4Project::updateCppCodeModel()
     QStringList predefinedFrameworkPaths;
     QByteArray predefinedMacros;
 
-    QString qtFrameworkPath = activeBC->qtVersion()->frameworkInstallPath();
+    QString qtFrameworkPath;
+    if (activeBC->qtVersion())
+        qtFrameworkPath = activeBC->qtVersion()->frameworkInstallPath();
     if (!qtFrameworkPath.isEmpty())
         predefinedFrameworkPaths.append(qtFrameworkPath);
 
@@ -435,7 +507,8 @@ void Qt4Project::updateCppCodeModel()
         predefinedMacros = tc->predefinedMacros();
 
         QList<HeaderPath> headers = tc->systemHeaderPaths();
-        headers.append(activeBC->qtVersion()->systemHeaderPathes());
+        if (activeBC->qtVersion())
+            headers.append(activeBC->qtVersion()->systemHeaderPathes());
         foreach (const HeaderPath &headerPath, headers) {
             if (headerPath.kind() == HeaderPath::FrameworkHeaderPath)
                 predefinedFrameworkPaths.append(headerPath.path());
@@ -478,26 +551,13 @@ void Qt4Project::updateCppCodeModel()
             if (!allIncludePaths.contains(includePath))
                 allIncludePaths.append(includePath);
         }
-
-#if 0 // Experimental PKGCONFIG support
-        { // Pkg Config support
-            QStringList pkgConfig = pro->variableValue(PkgConfigVar);
-            if (!pkgConfig.isEmpty()) {
-                pkgConfig.prepend("--cflags-only-I");
-                QProcess process;
-                process.start("pkg-config", pkgConfig);
-                process.waitForFinished();
-                QString result = process.readAllStandardOutput();
-                foreach(const QString &part, result.trimmed().split(' ', QString::SkipEmptyParts)) {
-                    info.includes.append(part.mid(2)); // Chop off "-I"
-                }
-            }
-        }
-#endif
     }
 
     // Add mkspec directory
-    allIncludePaths.append(activeBC->qtVersion()->mkspecPath());
+    if (rootQt4ProjectNode())
+        allIncludePaths.append(rootQt4ProjectNode()->resolvedMkspecPath());
+    else if (activeBC->qtVersion())
+        allIncludePaths.append(activeBC->qtVersion()->mkspecPath());
 
     allIncludePaths.append(predefinedIncludePaths);
 
@@ -517,7 +577,8 @@ void Qt4Project::updateCppCodeModel()
         && pinfo.includePaths == allIncludePaths
         && pinfo.frameworkPaths == allFrameworkPaths
         && fileList
-        && pinfo.precompiledHeaders == allPrecompileHeaders) {
+        && pinfo.precompiledHeaders == allPrecompileHeaders
+        && !m_codeModelCanceled) {
         // Nothing to update...
     } else {
         pinfo.sourceFiles.clear();
@@ -538,6 +599,7 @@ void Qt4Project::updateCppCodeModel()
 
         modelmanager->updateProjectInfo(pinfo);
         m_codeModelFuture = modelmanager->updateSourceFiles(pinfo.sourceFiles);
+        m_codeModelCanceled = false;
     }
 }
 
@@ -553,22 +615,38 @@ void Qt4Project::updateQmlJSCodeModel()
     FindQt4ProFiles findQt4ProFiles;
     QList<Qt4ProFileNode *> proFiles = findQt4ProFiles(rootProjectNode());
 
+    projectInfo.importPaths.clear();
     foreach (Qt4ProFileNode *node, proFiles) {
         projectInfo.importPaths.append(node->variableValue(QmlImportPathVar));
     }
+
     bool preferDebugDump = false;
+    projectInfo.tryQmlDump = false;
     if (activeTarget() && activeTarget()->activeBuildConfiguration()) {
-        preferDebugDump = activeTarget()->activeBuildConfiguration()->qmakeBuildConfiguration() & QtVersion::DebugBuild;
-        const QtVersion *qtVersion = activeTarget()->activeBuildConfiguration()->qtVersion();
-        if (qtVersion->isValid()) {
-            const QString qtVersionImportPath = qtVersion->versionInfo().value("QT_INSTALL_IMPORTS");
-            if (!qtVersionImportPath.isEmpty())
-                projectInfo.importPaths += qtVersionImportPath;
+        preferDebugDump = activeTarget()->activeQt4BuildConfiguration()->qmakeBuildConfiguration() & QtSupport::BaseQtVersion::DebugBuild;
+        QtSupport::BaseQtVersion *qtVersion = activeTarget()->activeQt4BuildConfiguration()->qtVersion();
+        if (qtVersion && qtVersion->isValid()) {
+            projectInfo.tryQmlDump = qtVersion->type() == QtSupport::Constants::DESKTOPQT
+                    || qtVersion->type() == QtSupport::Constants::SIMULATORQT;
+            projectInfo.qtImportsPath = qtVersion->versionInfo().value("QT_INSTALL_IMPORTS");
+            if (!projectInfo.qtImportsPath.isEmpty())
+                projectInfo.importPaths += projectInfo.qtImportsPath;
+            projectInfo.qtVersionString = qtVersion->qtVersionString();
         }
     }
-    QmlDumpTool::pathAndEnvironment(this, preferDebugDump, &projectInfo.qmlDumpPath, &projectInfo.qmlDumpEnvironment);
     projectInfo.importPaths.removeDuplicates();
 
+    if (projectInfo.tryQmlDump) {
+        const Qt4BuildConfiguration *bc = activeTarget()->activeQt4BuildConfiguration();
+        if (bc) {
+            QtSupport::QmlDumpTool::pathAndEnvironment(this, bc->qtVersion(), bc->toolChain(),
+                                                       preferDebugDump, &projectInfo.qmlDumpPath, &projectInfo.qmlDumpEnvironment);
+        }
+    } else {
+        projectInfo.qmlDumpPath.clear();
+        projectInfo.qmlDumpEnvironment.clear();
+    }
+
     modelManager->updateProjectInfo(projectInfo);
 }
 
@@ -584,7 +662,8 @@ void Qt4Project::update()
     if (debug)
         qDebug()<<"State is now Base";
     m_asyncUpdateState = Base;
-    activeTarget()->activeBuildConfiguration()->setEnabled(true);
+    activeTarget()->activeQt4BuildConfiguration()->setEnabled(true);
+    emit proParsingDone();
 }
 
 void Qt4Project::scheduleAsyncUpdate(Qt4ProFileNode *node)
@@ -593,7 +672,7 @@ void Qt4Project::scheduleAsyncUpdate(Qt4ProFileNode *node)
         return;
 
     if (debug)
-        qDebug()<<"schduleAsyncUpdate (node)";
+        qDebug()<<"schduleAsyncUpdate (node)"<<node->path();
     Q_ASSERT(m_asyncUpdateState != NoState);
 
     if (m_cancelEvaluate) {
@@ -605,7 +684,7 @@ void Qt4Project::scheduleAsyncUpdate(Qt4ProFileNode *node)
         return;
     }
 
-    activeTarget()->activeBuildConfiguration()->setEnabled(false);
+    activeTarget()->activeQt4BuildConfiguration()->setEnabled(false);
 
     if (m_asyncUpdateState == AsyncFullUpdatePending) {
         // Just postpone
@@ -619,7 +698,7 @@ void Qt4Project::scheduleAsyncUpdate(Qt4ProFileNode *node)
         // Add the node
         m_asyncUpdateState = AsyncPartialUpdatePending;
 
-        QList<Internal::Qt4ProFileNode *>::iterator it;
+        QList<Qt4ProFileNode *>::iterator it;
         bool add = true;
         if (debug)
             qDebug()<<"scheduleAsyncUpdate();"<<m_partialEvaluate.size()<<"nodes";
@@ -629,10 +708,10 @@ void Qt4Project::scheduleAsyncUpdate(Qt4ProFileNode *node)
                 add = false;
                 break;
             } else if (node->isParent(*it)) { // We already have the parent in the list, nothing to do
+                it = m_partialEvaluate.erase(it);
+            } else if ((*it)->isParent(node)) { // The node is the parent of a child already in the list
                 add = false;
                 break;
-            } else if ((*it)->isParent(node)) { // The node is the parent of a child already in the list
-                it = m_partialEvaluate.erase(it);
             } else {
                 ++it;
             }
@@ -675,21 +754,22 @@ void Qt4Project::scheduleAsyncUpdate()
             qDebug()<<"  update in progress, canceling and setting state to full update pending";
         m_cancelEvaluate = true;
         m_asyncUpdateState = AsyncFullUpdatePending;
-        activeTarget()->activeBuildConfiguration()->setEnabled(false);
-        m_rootProjectNode->emitProFileInvalidated();
+        activeTarget()->activeQt4BuildConfiguration()->setEnabled(false);
+        m_rootProjectNode->setParseInProgressRecursive(true);
         return;
     }
 
     if (debug)
         qDebug()<<"  starting timer for full update, setting state to full update pending";
     m_partialEvaluate.clear();
-    activeTarget()->activeBuildConfiguration()->setEnabled(false);
-    m_rootProjectNode->emitProFileInvalidated();
+    activeTarget()->activeQt4BuildConfiguration()->setEnabled(false);
+    m_rootProjectNode->setParseInProgressRecursive(true);
     m_asyncUpdateState = AsyncFullUpdatePending;
     m_asyncUpdateTimer.start();
 
     // Cancel running code model update
     m_codeModelFuture.cancel();
+    m_codeModelCanceled = true;
 }
 
 
@@ -729,14 +809,15 @@ void Qt4Project::decrementPendingEvaluateFutures()
             m_asyncUpdateTimer.start();
         } else  if (m_asyncUpdateState != ShuttingDown){
             // After being done, we need to call:
-            activeTarget()->activeBuildConfiguration()->setEnabled(true);
+            m_asyncUpdateState = Base;
+            activeTarget()->activeQt4BuildConfiguration()->setEnabled(true);
             foreach (Target *t, targets())
                 static_cast<Qt4BaseTarget *>(t)->createApplicationProFiles();
             updateFileList();
             updateCodeModels();
+            emit proParsingDone();
             if (debug)
                 qDebug()<<"  Setting state to Base";
-            m_asyncUpdateState = Base;
         }
     }
 }
@@ -746,19 +827,6 @@ bool Qt4Project::wasEvaluateCanceled()
     return m_cancelEvaluate;
 }
 
-QString Qt4Project::defaultTopLevelBuildDirectory() const
-{
-    return defaultTopLevelBuildDirectory(file()->fileName());
-}
-
-QString Qt4Project::defaultTopLevelBuildDirectory(const QString &profilePath)
-{
-    if (profilePath.isEmpty())
-        return QString();
-    QFileInfo info(profilePath);
-    return QDir::cleanPath(projectDirectory(profilePath) + QLatin1String("/../") + info.baseName() + QLatin1String("-build"));
-}
-
 void Qt4Project::asyncUpdate()
 {
     if (debug)
@@ -881,58 +949,75 @@ void Qt4Project::proFileParseError(const QString &errorMessage)
     Core::ICore::instance()->messageManager()->printToOutputPanePopup(errorMessage);
 }
 
-ProFileReader *Qt4Project::createProFileReader(Qt4ProFileNode *qt4ProFileNode, Qt4BuildConfiguration *bc)
+QtSupport::ProFileReader *Qt4Project::createProFileReader(Qt4ProFileNode *qt4ProFileNode, Qt4BuildConfiguration *bc)
 {
     if (!m_proFileOption) {
         m_proFileOption = new ProFileOption;
         m_proFileOptionRefCnt = 0;
 
         if (!bc && activeTarget())
-            bc = activeTarget()->activeBuildConfiguration();
+            bc = activeTarget()->activeQt4BuildConfiguration();
 
         if (bc) {
-            QtVersion *version = bc->qtVersion();
-            if (version->isValid()) {
+            QtSupport::BaseQtVersion *version = bc->qtVersion();
+            if (version && version->isValid()) {
                 m_proFileOption->properties = version->versionInfo();
                 if (bc->toolChain())
                     m_proFileOption->sysroot = bc->qtVersion()->systemRoot();
             }
 
+            Utils::Environment env = bc->environment();
+            Utils::Environment::const_iterator eit = env.constBegin(), eend = env.constEnd();
+            for (; eit != eend; ++eit)
+                m_proFileOption->environment.insert(env.key(eit), env.value(eit));
+
             QStringList args;
-            if (QMakeStep *qs = bc->qmakeStep())
+            if (QMakeStep *qs = bc->qmakeStep()) {
                 args = qs->parserArguments();
-            else
+                m_proFileOption->qmakespec = qs->mkspec();
+            } else {
                 args = bc->configCommandLineArguments();
+            }
             m_proFileOption->setCommandLineArguments(args);
         }
 
-        ProFileCacheManager::instance()->incRefCount();
+        QtSupport::ProFileCacheManager::instance()->incRefCount();
     }
     ++m_proFileOptionRefCnt;
 
-    ProFileReader *reader = new ProFileReader(m_proFileOption);
+    QtSupport::ProFileReader *reader = new QtSupport::ProFileReader(m_proFileOption);
 
     reader->setOutputDir(qt4ProFileNode->buildDir());
 
     return reader;
 }
 
-void Qt4Project::destroyProFileReader(ProFileReader *reader)
+ProFileOption *Qt4Project::proFileOption()
+{
+    return m_proFileOption;
+}
+
+void Qt4Project::destroyProFileReader(QtSupport::ProFileReader *reader)
 {
     delete reader;
     if (!--m_proFileOptionRefCnt) {
         QString dir = QFileInfo(m_fileInfo->fileName()).absolutePath();
         if (!dir.endsWith(QLatin1Char('/')))
             dir += QLatin1Char('/');
-        ProFileCacheManager::instance()->discardFiles(dir);
-        ProFileCacheManager::instance()->decRefCount();
+        QtSupport::ProFileCacheManager::instance()->discardFiles(dir);
+        QtSupport::ProFileCacheManager::instance()->decRefCount();
 
         delete m_proFileOption;
         m_proFileOption = 0;
     }
 }
 
-Qt4ProFileNode *Qt4Project::rootProjectNode() const
+ProjectExplorer::ProjectNode *Qt4Project::rootProjectNode() const
+{
+    return m_rootProjectNode;
+}
+
+Qt4ProFileNode *Qt4Project::rootQt4ProjectNode() const
 {
     return m_rootProjectNode;
 }
@@ -945,6 +1030,14 @@ bool Qt4Project::validParse(const QString &proFilePath) const
     return node && node->validParse();
 }
 
+bool Qt4Project::parseInProgress(const QString &proFilePath) const
+{
+    if (!m_rootProjectNode)
+        return false;
+    const Qt4ProFileNode *node = m_rootProjectNode->findProFileFor(proFilePath);
+    return node && node->parseInProgress();
+}
+
 QList<BuildConfigWidget*> Qt4Project::subConfigWidgets()
 {
     QList<BuildConfigWidget*> subWidgets;
@@ -965,8 +1058,8 @@ void Qt4Project::collectAllfProFiles(QList<Qt4ProFileNode *> &list, Qt4ProFileNo
 
 void Qt4Project::collectApplicationProFiles(QList<Qt4ProFileNode *> &list, Qt4ProFileNode *node)
 {
-    if (node->projectType() == Internal::ApplicationTemplate
-        || node->projectType() == Internal::ScriptTemplate) {
+    if (node->projectType() == ApplicationTemplate
+        || node->projectType() == ScriptTemplate) {
         list.append(node);
     }
     foreach (ProjectNode *n, node->subProjectNodes()) {
@@ -981,7 +1074,7 @@ QList<Qt4ProFileNode *> Qt4Project::allProFiles() const
     QList<Qt4ProFileNode *> list;
     if (!rootProjectNode())
         return list;
-    collectAllfProFiles(list, rootProjectNode());
+    collectAllfProFiles(list, rootQt4ProjectNode());
     return list;
 }
 
@@ -990,7 +1083,7 @@ QList<Qt4ProFileNode *> Qt4Project::applicationProFiles() const
     QList<Qt4ProFileNode *> list;
     if (!rootProjectNode())
         return list;
-    collectApplicationProFiles(list, rootProjectNode());
+    collectApplicationProFiles(list, rootQt4ProjectNode());
     return list;
 }
 
@@ -1051,17 +1144,28 @@ void Qt4Project::notifyChanged(const QString &name)
 {
     if (files(Qt4Project::ExcludeGeneratedFiles).contains(name)) {
         QList<Qt4ProFileNode *> list;
-        findProFile(name, rootProjectNode(), list);
+        findProFile(name, rootQt4ProjectNode(), list);
         foreach(Qt4ProFileNode *node, list) {
-            ProFileCacheManager::instance()->discardFile(name);
+            QtSupport::ProFileCacheManager::instance()->discardFile(name);
             node->update();
         }
+        updateFileList();
     }
 }
 
-CentralizedFolderWatcher *Qt4Project::centralizedFolderWatcher()
+void Qt4Project::watchFolders(const QStringList &l, Qt4PriFileNode *node)
 {
-    return &m_centralizedFolderWatcher;
+    if (l.isEmpty())
+        return;
+    if (!m_centralizedFolderWatcher)
+        m_centralizedFolderWatcher = new Internal::CentralizedFolderWatcher(this);
+    m_centralizedFolderWatcher->watchFolders(l, node);
+}
+
+void Qt4Project::unwatchFolders(const QStringList &l, Qt4PriFileNode *node)
+{
+    if (m_centralizedFolderWatcher && !l.isEmpty())
+        m_centralizedFolderWatcher->unwatchFolders(l, node);
 }
 
 /////////////
@@ -1074,7 +1178,7 @@ namespace {
    bool debugCFW = false;
 }
 
-CentralizedFolderWatcher::CentralizedFolderWatcher()
+CentralizedFolderWatcher::CentralizedFolderWatcher(QObject *parent) : QObject(parent)
 {
     m_compressTimer.setSingleShot(true);
     m_compressTimer.setInterval(200);
@@ -1260,4 +1364,6 @@ void CentralizedFolderWatcher::delayedFolderChanged(const QString &folder)
 //    }
 //}
 
+} // namespace Qt4ProjectManager
 
+#include "qt4project.moc"
