@@ -4,7 +4,7 @@
 **
 ** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
 **
-** Contact: Nokia Corporation (info@qt.nokia.com)
+** Contact: Nokia Corporation (qt-info@nokia.com)
 **
 **
 ** GNU Lesser General Public License Usage
@@ -33,12 +33,10 @@
 #ifndef QT4PROJECT_H
 #define QT4PROJECT_H
 
-#include "qt4nodes.h"
-#include "qt4target.h"
+#include "qt4projectmanager_global.h"
 
 #include <projectexplorer/project.h>
 #include <projectexplorer/projectnodes.h>
-#include <coreplugin/ifile.h>
 
 #include <QtCore/QStringList>
 #include <QtCore/QMap>
@@ -50,18 +48,23 @@ QT_BEGIN_NAMESPACE
 struct ProFileOption;
 QT_END_NAMESPACE
 
+namespace QtSupport {
+class ProFileReader;
+}
+
 namespace Qt4ProjectManager {
+class Qt4ProFileNode;
+class Qt4PriFileNode;
+class Qt4BaseTarget;
+class Qt4BuildConfiguration;
 
 namespace Internal {
-    class ProFileReader;
     class DeployHelperRunStep;
     class FileItem;
-    class Qt4ProFileNode;
-    class Qt4PriFileNode;
     class GCCPreprocessor;
     struct Qt4ProjectFiles;
     class Qt4ProjectConfigWidget;
-
+    class Qt4ProjectFile;
     class Qt4NodesWatcher;
 }
 
@@ -72,65 +75,11 @@ class Qt4Manager;
 class Qt4Project;
 class Qt4RunStep;
 
-class Qt4ProjectFile : public Core::IFile
-{
-    Q_OBJECT
-
-public:
-    Qt4ProjectFile(Qt4Project *project, const QString &filePath, QObject *parent = 0);
-
-    bool save(const QString &fileName = QString());
-    QString fileName() const;
-    virtual void rename(const QString &newName);
-
-    QString defaultPath() const;
-    QString suggestedFileName() const;
-    virtual QString mimeType() const;
-
-    bool isModified() const;
-    bool isReadOnly() const;
-    bool isSaveAsAllowed() const;
-
-    ReloadBehavior reloadBehavior(ChangeTrigger state, ChangeType type) const;
-    void reload(ReloadFlag flag, ChangeType type);
-
-private:
-    const QString m_mimeType;
-    Qt4Project *m_project;
-    QString m_filePath;
-};
-
-/// Watches folders for Qt4PriFile nodes
-/// use one file system watcher to watch all folders
-/// such minimizing system ressouce usage
 namespace Internal {
-class CentralizedFolderWatcher : public QObject
-{
-    Q_OBJECT
-public:
-    CentralizedFolderWatcher();
-    ~CentralizedFolderWatcher();
-    void watchFolders(const QList<QString> &folders, Qt4PriFileNode *node);
-    void unwatchFolders(const QList<QString> &folders, Qt4PriFileNode *node);
-
-private slots:
-    void folderChanged(const QString &folder);
-    void onTimer();
-    void delayedFolderChanged(const QString &folder);
-
-private:
-    QSet<QString> recursiveDirs(const QString &folder);
-    QFileSystemWatcher m_watcher;
-    QMultiMap<QString, Qt4PriFileNode *> m_map;
-
-    QSet<QString> m_recursiveWatchedFolders;
-    QTimer m_compressTimer;
-    QSet<QString> m_changedFolders;
-};
-
+class CentralizedFolderWatcher;
 }
 
-class Qt4Project : public ProjectExplorer::Project
+class  QT4PROJECTMANAGER_EXPORT Qt4Project : public ProjectExplorer::Project
 {
     Q_OBJECT
 
@@ -149,28 +98,32 @@ public:
     QList<Core::IFile *> dependencies();     //NBS remove
     QList<ProjectExplorer::Project *>dependsOn();
 
-    Internal::Qt4ProFileNode *rootProjectNode() const;
+    ProjectExplorer::ProjectNode *rootProjectNode() const;
+    Qt4ProFileNode *rootQt4ProjectNode() const;
     bool validParse(const QString &proFilePath) const;
+    bool parseInProgress(const QString &proFilePath) const;
 
     virtual QStringList files(FilesMode fileMode) const;
     virtual QString generatedUiHeader(const QString &formFile) const;
 
     QList<ProjectExplorer::BuildConfigWidget*> subConfigWidgets();
 
-    QList<Internal::Qt4ProFileNode *> allProFiles() const;
-    QList<Internal::Qt4ProFileNode *> applicationProFiles() const;
+    QList<Qt4ProFileNode *> allProFiles() const;
+    QList<Qt4ProFileNode *> applicationProFiles() const;
     bool hasApplicationProFile(const QString &path) const;
     QStringList applicationProFilePathes(const QString &prepend = QString()) const;
 
     void notifyChanged(const QString &name);
 
     /// \internal
-    Internal::ProFileReader *createProFileReader(Internal::Qt4ProFileNode *qt4ProFileNode, Qt4BuildConfiguration *bc = 0);
+    QtSupport::ProFileReader *createProFileReader(Qt4ProFileNode *qt4ProFileNode, Qt4BuildConfiguration *bc = 0);
     /// \internal
-    void destroyProFileReader(Internal::ProFileReader *reader);
+    ProFileOption *proFileOption();
+    /// \internal
+    void destroyProFileReader(QtSupport::ProFileReader *reader);
 
     /// \internal
-    void scheduleAsyncUpdate(Qt4ProjectManager::Internal::Qt4ProFileNode *node);
+    void scheduleAsyncUpdate(Qt4ProjectManager::Qt4ProFileNode *node);
     /// \internal
     void incrementPendingEvaluateFutures();
     /// \internal
@@ -178,17 +131,16 @@ public:
     /// \internal
     bool wasEvaluateCanceled();
 
-    QString defaultTopLevelBuildDirectory() const;
-    static QString defaultTopLevelBuildDirectory(const QString &profilePath);
-
-    Internal::CentralizedFolderWatcher *centralizedFolderWatcher();
-
     // For Qt4ProFileNode after a on disk change
     void updateFileList();
+    void updateCodeModels();
+
+    void watchFolders(const QStringList &l, Qt4PriFileNode *node);
+    void unwatchFolders(const QStringList &l, Qt4PriFileNode *node);
 
 signals:
-    void proFileUpdated(Qt4ProjectManager::Internal::Qt4ProFileNode *node, bool);
-    void proFileInvalidated(Qt4ProjectManager::Internal::Qt4ProFileNode *node);
+    void proParsingDone();
+    void proFileUpdated(Qt4ProjectManager::Qt4ProFileNode *node, bool, bool);
     void buildDirectoryInitialized();
     void fromMapFinished();
 
@@ -210,15 +162,14 @@ private slots:
 private:
     void scheduleAsyncUpdate();
 
-    void updateCodeModels();
     void updateCppCodeModel();
     void updateQmlJSCodeModel();
 
 
-    static void collectAllfProFiles(QList<Internal::Qt4ProFileNode *> &list, Internal::Qt4ProFileNode *node);
-    static void collectApplicationProFiles(QList<Internal::Qt4ProFileNode *> &list, Internal::Qt4ProFileNode *node);
-    static void findProFile(const QString& fileName, Internal::Qt4ProFileNode *root, QList<Internal::Qt4ProFileNode *> &list);
-    static bool hasSubNode(Internal::Qt4PriFileNode *root, const QString &path);
+    static void collectAllfProFiles(QList<Qt4ProFileNode *> &list, Qt4ProFileNode *node);
+    static void collectApplicationProFiles(QList<Qt4ProFileNode *> &list, Qt4ProFileNode *node);
+    static void findProFile(const QString& fileName, Qt4ProFileNode *root, QList<Qt4ProFileNode *> &list);
+    static bool hasSubNode(Qt4PriFileNode *root, const QString &path);
 
     static bool equalFileList(const QStringList &a, const QStringList &b);
 
@@ -227,10 +178,10 @@ private:
     static QString qmakeVarName(ProjectExplorer::FileType type);
 
     Qt4Manager *m_manager;
-    Internal::Qt4ProFileNode *m_rootProjectNode;
+    Qt4ProFileNode *m_rootProjectNode;
     Internal::Qt4NodesWatcher *m_nodesWatcher;
 
-    Qt4ProjectFile *m_fileInfo;
+    Internal::Qt4ProjectFile *m_fileInfo;
 
     // Current configuration
     QString m_oldQtIncludePath;
@@ -249,13 +200,14 @@ private:
     enum AsyncUpdateState { NoState, Base, AsyncFullUpdatePending, AsyncPartialUpdatePending, AsyncUpdateInProgress, ShuttingDown };
     AsyncUpdateState m_asyncUpdateState;
     bool m_cancelEvaluate;
-    QList<Internal::Qt4ProFileNode *> m_partialEvaluate;
+    bool m_codeModelCanceled;
+    QList<Qt4ProFileNode *> m_partialEvaluate;
 
     QFuture<void> m_codeModelFuture;
 
-    Internal::CentralizedFolderWatcher m_centralizedFolderWatcher;
+    Internal::CentralizedFolderWatcher *m_centralizedFolderWatcher;
 
-    friend class Qt4ProjectFile;
+    friend class Internal::Qt4ProjectFile;
     friend class Internal::Qt4ProjectConfigWidget;
 };
 
